/*
 * Copyright (C) 2021 Tycho Softworks.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef	CALLS_HPP_
#define CALLS_HPP_

#include "features.hpp"

namespace server {
class registry;
class event_t;

using reason_t = struct reason final {
    explicit reason(int exit_code = 487, std::string exit_text = "") :
    code(exit_code), text(std::move(exit_text)) {}
    int code;
    std::string text;
};

// TODO: future tasks
struct session final {
public:
    static auto next_callid() -> uint64_t;
    static void startup(const keyfile& config);
    static void reload(const keyfile& config);
    static void hangup_call(uint64_t id, const reason_t& reason);
    static void hangup_user(const registry& user, const reason_t& reason);
    static void close_invite(const event_t& event);
    static void trying_invite(const event_t& event);
    static void release_invite(const event_t& event);
    static void answer_invite(const event_t& event);
    static void ringing_invite(const event_t& event);
    static void failed_invite(const event_t& event);
    static void redo_invite(const event_t& event);
    static void ack_invite(const event_t& event);
    static void call_info(const event_t& event);
    static void call_server(uint64_t id, const event_t& event, const feature_t& f);
    static void create_call(uint64_t id, const event_t& event, const registry& entry);
    static void invite_call(uint64_t id, const registry& entry, unsigned rings = 0);
    static void invite_delayed(uint64_t id, const registry& entry, unsigned rings = 0);
    static void start_call(uint64_t id, const event_t& event, const registry& entry);
    static void shutdown();
};
} // end namespace
#endif
