/*
 * Copyright (C) 2023 Tycho Softworks.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef FEATURES_HPP_
#define FEATURES_HPP_

#include "events.hpp"
#include "registry.hpp"
#include "utils.hpp"
#include "exosip.hpp"

namespace server {
struct feature final {
    std::string code;           // as dialed
    mutable std::string name;   // name of feature
    mutable std::string extra;  // extra digits if any
    registry server, origin;
    bool sysop{false};

    static void startup(const keyfile& config);
    static void reload(const keyfile& config);
    static void dialed(std::shared_ptr<const feature> f, const event_t &event);
    static void shutdown();
};

using feature_data = feature;
using feature_t = std::shared_ptr<const feature>;
} // end namespace
#endif
